package net.sf.corn.converter.json;


import static net.sf.corn.converter.json.IJsonConstants.ATTR_CLASSNAME;
import static net.sf.corn.converter.json.IJsonConstants.CHAR_CLASS_END;
import static net.sf.corn.converter.json.IJsonConstants.CHAR_CLASS_START;
import static net.sf.corn.converter.json.IJsonConstants.CHAR_FIELD_NAME__VALUE_SEPERATOR;
import static net.sf.corn.converter.json.IJsonConstants.CHAR_FIELD_SEPERATOR;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import net.sf.corn.converter.ConversionException;

/**
 * A Complex Json Object may contains multiple Json Objects within it
 * @author Serhat Dirik
 */
public class JsTypeComplex extends JsTypeObject implements Serializable {

	private static final long serialVersionUID = 1250449226328996498L;
	
	private ConcurrentHashMap<String, JsTypeObject> map = new ConcurrentHashMap<String, JsTypeObject>();
	
	/**
	 * Constructs a new complex json object
	 */
	public JsTypeComplex() {
	}
	
	/**
	 * Constructs a new complex json object and uses the name of the given class
	 * @param clazz a java class that name to be used as class name in this complex object
	 */
	public JsTypeComplex(Class<?> clazz) {
		this(clazz != null ? clazz.getCanonicalName() : null , null);
	}

	/**
	 * Constructs a new object with the given java class name and 
	 * imports the given map as attributes
	 * @param className
	 * @param map
	 */
	public JsTypeComplex(String className,
			Map<String, JsTypeObject> map) {
		setJavaClassName(className);
		if(isArray()){
			throw new IllegalArgumentException("Array class "+className+ " can't be used for JsonComplex");
		}
		if (map != null && map.size() > 0) {
			this.map.putAll(map);
		}
	}

	@Override
	public String toJsonString() {

		StringBuilder sb = new StringBuilder();

		sb.append(CHAR_CLASS_START);
		if (getJavaClassName() != null) {
			sb.append(quote(ATTR_CLASSNAME))
					.append(CHAR_FIELD_NAME__VALUE_SEPERATOR)
					.append(quote(getJavaClassName()));
		}
		boolean first = true;
		for (String key : keys()) {
			if (!first || getJavaClassName() != null) {
				sb.append(CHAR_FIELD_SEPERATOR);
			}
			first = false;
			sb.append(quote(key)).append(CHAR_FIELD_NAME__VALUE_SEPERATOR)
					.append(map.get(key).toString());
		}
		sb.append(CHAR_CLASS_END);

		return sb.toString();
	}
	
	@Override
	public String toString() {
		return toJsonString();
	}

	@Override
	public String toStringValue() {
		return map.toString();
	}

	@Override
	public boolean isNull() {
		return false;
	}

	@Override
	public boolean isComplex() {
		return true;
	}

	@Override
	public boolean isList() {
		return false;
	}
	
	/**
	 * Gives the value of the contained object that mapped with the given key
	 * @param key  name of the attribute
	 * @return gives the requested attribute value or null if there is no mapped attribute
	 */
	public JsTypeObject get(String key) {
		return map != null ? map.get(key) : null;
	}
	/**
	 * Puts the given object as an attribute in the complex object
	 * @param key name of the attribute
	 * @param val Json object typed value
	 * @return returns null value it is set the first time, otherwise gives the previous value
	 */
	public JsTypeObject put(String key, JsTypeObject val) {
		JsTypeObject ret = map.put(key, val);
		return ret;
	}
	/**
	 * Puts the given java object as an attribute in the complex object by converting it to a Json Object
	 * @param key name of the attribute
	 * @param val Json object typed value
	 * @return returns null value it is set the first time, otherwise gives the previous value
	 */
	public JsTypeObject putObject(String key, Object val) throws ConversionException {
		if(val instanceof JsTypeObject){
			return put(key, (JsTypeObject) val);
		}else{
			JsonConverter converter = new JsonConverter();
			JsTypeObject ret = map.put(key, converter.toTargetType(val));
			return ret;
		}
	}
	
	/**
	 * Gives list of the attribute names 
	 * @return attribute names as set
	 */
	public Set<String> keys() {
		return map == null ? new HashSet<String>() : map.keySet();
	}
	/**
	 * Tells if the complex contains the given attribute
	 * @param key name of the attribute
	 * @return true if the attribute contained
	 */
	public boolean has(String key) {
		return map == null ? false : map.containsKey(key);
	}

	/**
	 * Removes the given attribute
	 * @param key name of the attribute
	 * @return null if there is no value is set with the given name, otherwise returns the value
	 */
	public JsTypeObject remove(String key) {
		return map == null ? null : map.remove(key);
	}
	
	@Override
	public int size() {
		return map == null ? 0 : map.size();
	}
	@Override
	protected Object clone() throws CloneNotSupportedException {
		return new JsTypeComplex(getJavaClassName(), this.map);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((javaClassName == null) ? 0 : javaClassName.hashCode());
		result = prime * result + ((map == null) ? 0 : map.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		JsTypeComplex other = (JsTypeComplex) obj;
		if (javaClassName == null) {
			if (other.javaClassName != null)
				return false;
		} else if (!javaClassName.equals(other.javaClassName))
			return false;
		if (map == null) {
			if (other.map != null)
				return false;
		} else if (!map.equals(other.map))
			return false;
		return true;
	}


}
