package net.sf.corn.converter.json;

import static net.sf.corn.converter.json.IJsonConstants.*;

import java.io.Serializable;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import net.sf.corn.converter.ConversionException;

/**
 * Represents list and array types.  
 * @author Serhat Dirik
 */
public class JsTypeList extends JsTypeObject implements Serializable {
 
	private static final long serialVersionUID = -8703766326992443751L;
	private CopyOnWriteArrayList<JsTypeObject> list = new CopyOnWriteArrayList<>();
	
	/**
	 * Default constructor
	 */
	public JsTypeList(){
	}
	
	/**
	 * Constructs a new complex json object and uses the name of the given class
	 * @param clazz a java class that name to be used as class name in this complex object
	 */
	public JsTypeList(Class<?> clazz) {
		this(clazz != null ? clazz.getCanonicalName() : null , null);
	}
	
	/**
	 * Constructs a new object with the given java class name and 
	 * imports the given list
	 * @param className
	 * @param map
	 */
	public JsTypeList(String className, List<JsTypeObject> list) {
		setJavaClassName(className);
		if (list != null) {
			this.list.addAll(list);
		}

	}

	@Override
	public String toJsonString() {
		if (isNull())
			return NULL_STRING_VALUE;
		StringBuilder sb = new StringBuilder();
		if (isNull() && (getJavaClassName() == null))
			return NULL_STRING_VALUE;
		String className = getJavaClassName();
		if (className != null) {
			sb.append(CHAR_CLASS_START);
			sb.append(quote(ATTR_CLASSNAME))
					.append(CHAR_FIELD_NAME__VALUE_SEPERATOR)
					.append(quote(getJavaClassName()));
			sb.append(CHAR_FIELD_SEPERATOR).append(quote(ATTR_LIST))
					.append(CHAR_FIELD_NAME__VALUE_SEPERATOR);
		}
		sb.append(CHAR_ARRAY_START);
		int len = size();
		for (int i = 0; i < len; i += 1) {
			if (i > 0) {
				sb.append(CHAR_FIELD_SEPERATOR);
			}
			sb.append(list.get(i).toJsonString());
		}
		sb.append(CHAR_ARRAY_END);
		if (className != null) {
			sb.append(CHAR_CLASS_END);
		}

		return sb.toString();
	}
	@Override
	public String toString() {
		return toJsonString();
	}
	/**
	 * Adds the given object to the list
	 * @param object object to be added to the list 
	 * @return true if its successfully added
	 */
	public boolean add(JsTypeObject object) {
		return list.add(object);
	}
	/**
	 * Adds the given object to the list by converting to a Json Object
	 * @param object object to be added to the list 
	 * @return true if its successfully added
	 */
	public boolean addObject(Object object) throws ConversionException {
		if(object instanceof JsTypeObject){
			return add((JsTypeObject) object);
		}else{
			JsonConverter converter = new JsonConverter();
			return list.add(converter.toTargetType(object));
		}
	}
	/**
	 * Tells whether the given object contained in the list or not
	 * @param o json object to be checked 
	 * @return true  if the given object is contained in the list
	 */
	public boolean contains(JsTypeObject o) {
		return list.contains(o);
	}

	/**
	 * Gives the first occurrence of the given element in the list, or -1 if it's not contained it the list
	 * @param o the object to be checked
	 * @return index of the given object
	 */
	public int indexOf(JsTypeObject o) {
		return list.indexOf(o);
	}
	/**
	 * Gives the last occurrence of the given element in the list, or -1 if it's not contained it the list
	 * @param o the object to be checked
	 * @return index of the given object
	 */
	public int lastIndexOf(JsTypeObject o) {
		return list.lastIndexOf(o);
	}

	/**
	 *  Removes the element at the specified position in this list 
	 * @param index the position to be removed from the list
	 */
	public void remove(int index) {
		list.remove(index);
	}
	/**
	 * Removes the first occurrence of the specified element from this list, if it is present 
	 * @param o
	 * @return 
	 */
	public boolean remove(JsTypeObject o) {
		return list.remove(o);
	}
	/**
	 * Retrieves the object that saved in the given position
	 * @param index positive integer that less than or equals to the list size
	 * @return the object in the asked position
	 */
	public JsTypeObject get(int index) {
		return list.get(index);
	}
	
	@Override
	public int size() {
		return list == null ? 0 : list.size();

	}
	@Override
	protected Object clone() throws CloneNotSupportedException {
		CopyOnWriteArrayList<JsTypeObject> nlist = new CopyOnWriteArrayList<>();
		nlist.addAll(list);
		return new JsTypeList(getJavaClassName(), nlist);

	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((javaClassName == null) ? 0 : javaClassName.hashCode());
		result = prime * result + ((list == null) ? 0 : list.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		JsTypeList other = (JsTypeList) obj;
		if (javaClassName == null) {
			if (other.javaClassName != null)
				return false;
		} else if (!javaClassName.equals(other.javaClassName))
			return false;
		if (list == null) {
			if (other.list != null)
				return false;
		} else if (!list.equals(other.list))
			return false;
		return true;
	}

	@Override
	public String toStringValue() {
		return list.toString();
	}

	@Override
	public boolean isNull() {
		return false;
	}

	@Override
	public boolean isComplex() {
		return false;
	}

	@Override
	public boolean isList() {
		return true;
	}
}
